@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
-"""
-pysat.instruments.methods is a pysat module that provides
-specific functions for instruments and classes of instruments.
+"""Provides specific functions for instruments and classes of instruments.
+
 Each set of methods is contained within a subpackage of this set.
 """
 
