@@ -3,6 +3,7 @@
 # Full author list can be found in .zenodo.json file
 # DOI:10.5281/zenodo.1199703
 # ----------------------------------------------------------------------------
+"""Utilities for file management and parsing file names."""
 
 import collections
 import glob
@@ -14,18 +15,19 @@ import string
 
 import pandas as pds
 
-from pysat.utils.time import create_datetime_index
 from pysat.utils._core import available_instruments
+from pysat.utils._core import listify
+from pysat.utils.time import create_datetime_index
 
 
 def process_parsed_filenames(stored, two_digit_year_break=None):
-    """Create a Files pandas Series of filenames from a formatted dict
+    """Create a Files pandas Series of filenames from a formatted dict.
 
     Parameters
     ----------
     stored : collections.orderedDict
-        Dict produced by parse_fixed_width_filenames or
-        parse_delimited_filenames
+        Dict produced by `parse_fixed_width_filenames` or
+        `parse_delimited_filenames`
     two_digit_year_break : int or NoneType
         If filenames only store two digits for the year, then
         '1900' will be added for years >= two_digit_year_break
@@ -71,20 +73,21 @@ def process_parsed_filenames(stored, two_digit_year_break=None):
         rec_arr = np.rec.fromarrays(rec_arr, names=val_keys)
         rec_arr.sort(order=val_keys, axis=0)
 
-        # pull out sorted info
+        # Pull out sorted info
         for key in keys:
             stored[key] = rec_arr[key]
         files = rec_arr['files']
 
-        # add hour and minute information to 'second'
+        # Add hour and minute information to 'second'
         if stored['second'] is None:
             stored['second'] = np.zeros(len(files))
         if stored['hour'] is not None:
             stored['second'] += 3600 * stored['hour']
         if stored['minute'] is not None:
             stored['second'] += 60 * stored['minute']
-        # version shouldn't be set to zero
-        # version is required to remove duplicate datetimes
+
+        # The version shouldn't be set to zero, it is required to remove
+        # duplicate datetimes
         if stored['revision'] is None:
             stored['revision'] = np.zeros(len(files))
         if stored['cycle'] is None:
@@ -95,14 +98,12 @@ def process_parsed_filenames(stored, two_digit_year_break=None):
                                       day=stored['day'],
                                       uts=stored['second'])
 
-        # if version, revision, and cycle are supplied
-        # use these parameters to weed out files that have been replaced
-        # with updated versions
-        # first, check for duplicate index times
+        # If version, revision, and cycle are supplied, use these parameters
+        # to weed out files that have been replaced with updated versions.
+        # First, check for duplicate index times
         dups = index[index.duplicated()].unique()
         if (len(dups) > 0) and (stored['version'] is not None):
-            # we have duplicates
-            # keep the highest version/revision combo
+            # We have duplicates, keep the highest version/revision combo
             version = pds.Series(stored['version'], index=index)
             revision = pds.Series(stored['revision'], index=index)
             cycle = pds.Series(stored['cycle'], index=index)
@@ -115,189 +116,273 @@ def process_parsed_filenames(stored, two_digit_year_break=None):
 
             return frame['files']
         else:
-            return pds.Series(files, index=index)
+            return pds.Series(files, index=index).sort_index()
     else:
         return pds.Series(None, dtype='object')
 
 
 def parse_fixed_width_filenames(files, format_str):
-    """Parses list of files, extracting data identified by format_str
+    """Parse list of files, extracting data identified by `format_str`.
 
     Parameters
     ----------
     files : list
-        List of files
+        List of files, typically provided by
+        `files.search_local_system_formatted_filename`.
     format_str : str
         Provides the naming pattern of the instrument files and the
         locations of date information so an ordered list may be produced.
-        Supports string formatting codes 'year', 'month', 'day', 'hour',
-        'minute', 'second', 'version', 'revision', and 'cycle'. For example,
-        `instrument_{year:4d}{month:02d}{day:02d}_v{version:02d}.cdf`
+        Supports all provided string formatting codes though only 'year',
+        'month', 'day', 'hour', 'minute', 'second', 'version', 'revision',
+        and 'cycle' will be used for time and sorting information. For example,
+        `instrument-{year:4d}_{month:02d}-{day:02d}_v{version:02d}.cdf`, or
+        `*-{year:4d}_{month:02d}hithere{day:02d}_v{version:02d}.cdf`
 
     Returns
     -------
     stored : collections.OrderedDict
-        Information parsed from filenames that inclues: 'year', 'month', 'day',
-        'hour', 'minute', 'second', 'version', 'revision', and 'cycle'.  Also
-        includes 'files', an input list of files, and 'format_str', a formatted
-        string from input
+        Information parsed from filenames that includes: 'year', 'month', 'day',
+        'hour', 'minute', 'second', 'version', 'revision', and 'cycle', as
+        well as any other user provided template variables. Also
+        includes `files`, an input list of files, and `format_str`.
+
+    Note
+    ----
+    The function uses the lengths of the fixed characters within `format_str`,
+    as well as the supplied lengths for template variables, to determine
+    where to parse out information. Thus, support for the wildcard '*' is
+    limited to locations before the first template variable.
 
     """
 
-    # create storage for data to be parsed from filenames
+    # Create storage for data to be parsed from filenames
     ordered_keys = ['year', 'month', 'day', 'hour', 'minute', 'second',
                     'version', 'revision', 'cycle']
     stored = collections.OrderedDict({kk: list() for kk in ordered_keys})
 
     if len(files) == 0:
         stored['files'] = []
-        # include format string as convenience for later functions
+        # Include format string as convenience for later functions
         stored['format_str'] = format_str
         return stored
 
-    # parse format string to get information needed to parse filenames
+    # Parse format string to get information needed to parse filenames
     search_dict = construct_searchstring_from_format(format_str)
-    snips = search_dict['string_blocks']
-    lengths = search_dict['lengths']
-    keys = search_dict['keys']
 
-    # determine the locations the date/version information in a filename is
-    # stored use these indices to slice out date from filenames
+    # Add non-standard keys
+    for key in search_dict['keys']:
+        if key not in stored:
+            stored[key] = []
+
+    # Determine the locations the date/version information in a filename is
+    # stored and use these indices to slice out date from filenames.
     idx = 0
     begin_key = []
     end_key = []
-    for i, snip in enumerate(snips):
+    for i, snip in enumerate(search_dict['string_blocks']):
         idx += len(snip)
-        if i < (len(lengths)):
+        if i < len(search_dict['lengths']):
             begin_key.append(idx)
-            idx += lengths[i]
+            idx += search_dict['lengths'][i]
             end_key.append(idx)
     max_len = idx
-    # setting up negative indexing to pick out filenames
+
+    # Setting up negative indexing to pick out filenames
     key_str_idx = [np.array(begin_key, dtype=np.int64) - max_len,
                    np.array(end_key, dtype=np.int64) - max_len]
-    # need to parse out dates for datetime index
+
+    # Need to parse out dates for datetime index
     for i, temp in enumerate(files):
-        for j, key in enumerate(keys):
-            val = temp[key_str_idx[0][j]:key_str_idx[1][j]]
+        for j, key in enumerate(search_dict['keys']):
+            if key_str_idx[1][j] == 0:
+                # Last element is a variable to be parsed out
+                val = temp[key_str_idx[0][j]:]
+            else:
+                val = temp[key_str_idx[0][j]:key_str_idx[1][j]]
             stored[key].append(val)
-    # convert to numpy arrays
+
+    # Convert to numpy arrays
     for key in stored.keys():
-        stored[key] = np.array(stored[key]).astype(np.int64)
         if len(stored[key]) == 0:
             stored[key] = None
-    # include files in output
+        else:
+            try:
+                # Assume key value is numeric integer
+                stored[key] = np.array(stored[key]).astype(np.int64)
+            except ValueError:
+                # Store key value as string
+                stored[key] = np.array(stored[key])
+
+    # Include files in output
     stored['files'] = files
-    # include format string as convenience for later functions
+
+    # Include format string as convenience for later functions
     stored['format_str'] = format_str
 
     return stored
 
 
 def parse_delimited_filenames(files, format_str, delimiter):
-    """Parses list of files, extracting data identified by format_str
+    """Parse list of files, extracting data identified by format_str.
+
+    Will parse file using `delimiter` though the function does not require
+    every parsed item to be a variable, and more than one variable
+    may be within a parsed section. Thus, the main practical
+    difference with `parse_fixed_width_filenames` is more support for
+    the use of the wildcard '*' within `format_str`. Overuse
+    of the '*' wildcard increases the probability of false positive matches
+    if there are multiple instrument files in the directory.
 
     Parameters
     ----------
     files : list
-        List of files
+        List of files, typically provided by
+        `files.search_local_system_formatted_filename`.
     format_str : str
         Provides the naming pattern of the instrument files and the
         locations of date information so an ordered list may be produced.
-        Supports string formatting codes 'year', 'month', 'day', 'hour',
-        'minute', 'second', 'version', 'revision', and 'cycle'. For example,
-        `instrument_{year:4d}{month:02d}{day:02d}_v{version:02d}.cdf`
-    delimiter : string
+        Supports all provided string formatting codes though only 'year',
+        'month', 'day', 'hour', 'minute', 'second', 'version', 'revision',
+        and 'cycle' will be used for time and sorting information. For example,
+        `*_{year:4d}_{month:02d}_{day:02d}_*_v{version:02d}_*.cdf`
+    delimiter : str
         Delimiter string upon which files will be split (e.g., '.')
 
     Returns
     -------
     stored : collections.OrderedDict
-        Information parsed from filenames that inclues: 'year', 'month', 'day',
-        'hour', 'minute', 'second', 'version', 'revision', and 'cycle'.  Also
-        includes 'files', an input list of files, and 'format_str', a formatted
-        string from input
+        Information parsed from filenames that includes: 'year', 'month', 'day',
+        'hour', 'minute', 'second', 'version', 'revision', and 'cycle', as
+        well as any other user provided template variables. Also
+        includes `files`, an input list of files, and `format_str`.
+
+    Note
+    ----
+    The '*' wildcard is supported when leading, trailing, or wholly contained
+    between delimiters, such as 'data_name-{year:04d}-*-{day:02d}.txt',
+    or '*-{year:04d}*-*-{day:02d}*', where '-' is the delimiter.
+    There can not be a mixture of a template variable and '*' without a
+    delimiter in between, unless the '*' occurs after the variables. The
+    '*' should not be used to replace the delimited character in the filename.
 
     """
 
-    # create storage for data to be parsed from filenames
+    # Create storage for data to be parsed from filenames
     ordered_keys = ['year', 'month', 'day', 'hour', 'minute', 'second',
                     'version', 'revision', 'cycle']
-    stored = collections.OrderedDict({kk: list() for kk in ordered_keys})
+    stored = collections.OrderedDict({kk: None for kk in ordered_keys})
 
-    # exit early if there are no files
+    # Exit early if there are no files
     if len(files) == 0:
         stored['files'] = []
-        # include format string as convenience for later functions
+
+        # Include format string as convenience for later functions
         stored['format_str'] = format_str
         return stored
 
-    # parse format string to get information needed to parse filenames
-    search_dict = construct_searchstring_from_format(format_str, wildcard=True)
-    snips = search_dict['string_blocks']
-    keys = search_dict['keys']
+    # Parse format string to get information needed to parse filenames
+    search_dict = construct_searchstring_from_format(format_str, wildcard=False)
 
-    # going to parse string on delimiter
-    # it is possible that other regions have the delimiter but aren't
-    # going to be parsed out
-    # so apply delimiter breakdown to the string blocks as a guide
-    pblock = []
-    parsed_block = [snip.split(delimiter) for snip in snips]
-    for block in parsed_block:
-        if block != ['', '']:
-            if block[0] == '':
-                block = block[1:]
-            if block[-1] == '':
-                block = block[:-1]
-            pblock.extend(block)
-        pblock.append('')
-    parsed_block = pblock[:-1]
+    # Add non-standard keys
+    for key in search_dict['keys']:
+        if key not in stored:
+            stored[key] = None
+
+    # Going to parse the string on the delimiter. It is possible that other
+    # regions have the delimiter but aren't going to be parsed out.
+    # Reconstruct string from `snips` and use `{}` in place of `keys` and
+    # work from that.
+    recon = [''] * (len(search_dict['string_blocks'])
+                    + len(search_dict['keys']))
+    for i, item in enumerate(search_dict['string_blocks']):
+        recon[2 * i] = item
+    for i, item in enumerate(search_dict['keys']):
+        recon[2 * i + 1] = '{}'
+    recon = ''.join(recon)
+    split_recon = recon.split(delimiter)
+
+    # Parse out template variable information from reconstructed name.
+    # Store a list of indexes for locations to start pulling out
+    # variable information. For performance reasons, only want to do this
+    # once for `split_recon`.
+    split_idx = []
+    for i, rname in enumerate(split_recon):
+        loop_rname = rname
+
+        while True:
+            sidx = loop_rname.find('{}')
+            if sidx < 0:
+                # No template variables to parse
+                split_idx.append(None)
+                break
+            else:
+                # Found template variable marker. Remove marker and store
+                # location.
+                loop_rname = loop_rname[sidx + 2:]
+                split_idx.append(sidx)
+
+    # Parse out template variable information from each filename. Use the
+    # indices calculated above. First, prep memory.
+    for key in search_dict['keys']:
+        if stored[key] is None:
+            stored[key] = []
 
-    # need to parse out dates for datetime index
     for temp in files:
         split_name = temp.split(delimiter)
         idx = 0
-        for sname, bname in zip(split_name, parsed_block):
-            if bname == '':
-                # areas with data to be parsed are indicated with a
-                # '' in parsed_block
-                stored[keys[idx]].append(sname)
-                idx += 1
+        loop_split_idx = split_idx
+        for i, sname in enumerate(split_name):
+            loop_sname = sname
+            for j, sidx in enumerate(loop_split_idx):
+                if sidx is not None:
+                    # Pull out value from filename and shorten str.
+                    val = loop_sname[sidx:sidx + search_dict['lengths'][idx]]
+                    loop_sname = loop_sname[sidx + search_dict['lengths'][idx]:]
 
-    # convert to numpy arrays
+                    # Store parsed info and increment key index
+                    stored[search_dict['keys'][idx]].append(val)
+                    idx += 1
+                else:
+                    # No variable to be parsed, remove indices from
+                    # `loop_split_idx` already used.
+                    loop_split_idx = loop_split_idx[j + 1:]
+                    break
+
+    # Convert to numpy arrays
     for key in stored.keys():
-        try:
-            # Assume key value is numeric integer
-            stored[key] = np.array(stored[key]).astype(np.int64)
-        except ValueError:
-            # Store key value as string
-            stored[key] = np.array(stored[key])
-        if len(stored[key]) == 0:
-            stored[key] = None
+        if stored[key] is not None:
+            try:
+                # Assume key value is numeric integer
+                stored[key] = np.array(stored[key]).astype(np.int64)
+            except ValueError:
+                # Store key value as string
+                stored[key] = np.array(stored[key])
 
-    # include files in output
+    # Include files in output
     stored['files'] = files
 
-    # include format string as convenience for later functions
+    # Include format string as convenience for later functions
     stored['format_str'] = format_str
 
     return stored
 
 
 def construct_searchstring_from_format(format_str, wildcard=False):
-    """Parses format file string and returns string formatted for searching.
+    """Parse format file string and returns string formatted for searching.
+
+    Each variable in the string template is replaced with an appropriate
+    number of '?' based upon the provided length of the data.
 
     Parameters
     ----------
     format_str : str
         Provides the naming pattern of the instrument files and the
         locations of date information so an ordered list may be produced.
-        Supports 'year', 'month', 'day', 'hour', 'minute', 'second', 'version',
-        'revision', and 'cycle'. For example,
+        For example,
         `instrument_{year:04d}{month:02d}{day:02d}_v{version:02d}.cdf`
     wildcard : bool
-        If True, replaces the ? sequence with a * . This option may be well
-        suited when dealing with delimited filenames.
+        If True, replaces each '?' sequence that would normally
+        be returned with a single '*'.
 
     Returns
     -------
@@ -308,22 +393,28 @@ def construct_searchstring_from_format(format_str, wildcard=False):
         - 'lengths' (string length for data to be parsed)
         - 'string_blocks' (the filenames are broken into fixed width segments).
 
+    Raises
+    ------
+    ValueError
+        If a filename template isn't provided in `format_str`
+
     Note
     ----
     The '?' may be used to indicate a set number of spaces for a variable
     part of the name that need not be extracted.
     `cnofs_cindi_ivm_500ms_{year:4d}{month:02d}{day:02d}_v??.cdf`
 
-    A standards compliant filename can be constructed by starting with
-    string_blocks, adding keys in order, and replacing the '' locations
-    with data of length length.
+    A standards compliant filename can be constructed by adding the first
+    element from `string_blocks`, then the first item in `keys`, and iterating
+    that alternating pattern until all items are used.
+
+    This is the first function employed by `pysat.Files.from_os`.
 
     """
 
-    out_dict = {'search_string': '',
-                'keys': [],
-                'lengths': [],
+    out_dict = {'search_string': '', 'keys': [], 'lengths': [],
                 'string_blocks': []}
+
     if format_str is None:
         raise ValueError("Must supply a filename template (format_str).")
 
@@ -332,7 +423,7 @@ def construct_searchstring_from_format(format_str, wildcard=False):
     form = string.Formatter()
     for snip in form.parse(format_str):
         # Collect all of the format keywords. Replace them in the string with
-        # the '?' wildcard. The numnber of '?'s corresponds to the length of
+        # the '?' wildcard. The number of '?'s corresponds to the length of
         # data to be parsed. The length is obtained from format keywords so
         # that we know later on where to parse information out from.
         out_dict['search_string'] += snip[0]
@@ -342,39 +433,44 @@ def construct_searchstring_from_format(format_str, wildcard=False):
             out_dict['keys'].append(snip[1])
 
             # Try and determine formatting width
-            temp = re.findall(r'\d+', snip[2])
+            fwidths = re.findall(r'\d+', snip[2])
 
-            if temp:
+            if fwidths:
                 # There are items, try and grab width
-                for i in temp:
+                for fwidth in fwidths:
                     # Make sure there is truly something there
-                    if i != 0:
+                    if fwidth != 0:
                         # Store length and add to the search string
-                        out_dict['lengths'].append(np.int64(i))
+                        out_dict['lengths'].append(np.int64(fwidth))
                         if not wildcard:
-                            out_dict['search_string'] += '?' * np.int64(i)
+                            val = '?' * out_dict['lengths'][-1]
+                            out_dict['search_string'] += val
                         else:
                             out_dict['search_string'] += '*'
                         break
             else:
-                raise ValueError("Couldn't determine formatting width")
+                estr = ''.join(["Couldn't determine formatting width. ",
+                                "This may be due to the use of unsupported ",
+                                "wildcard characters."])
+                raise ValueError(estr)
 
     return out_dict
 
 
 def search_local_system_formatted_filename(data_path, search_str):
-    """
-    Parses format file string and returns string formatted for searching.
+    """Parse format file string and returns string formatted for searching.
 
     Parameters
     ----------
-    data_path : string
+    data_path : str
         Top level directory to search files for. This directory
         is provided by pysat to the instrument_module.list_files
         functions as data_path.
-    search_str : string
+    search_str : str
         String used to search for local files. For example,
         `cnofs_cindi_ivm_500ms_????????_v??.cdf` or `inst-name-*-v??.cdf`
+        Typically this input is provided by
+        `files.construct_searchstring_from_format`.
 
     Returns
     -------
@@ -402,7 +498,7 @@ def search_local_system_formatted_filename(data_path, search_str):
 def update_data_directory_structure(new_template, test_run=True,
                                     full_breakdown=False,
                                     remove_empty_dirs=False):
-    """Update pysat data directory structure to match supplied template
+    """Update pysat data directory structure to match supplied template.
 
     Translates all of pysat's managed science files to a new
     directory structure. By default, pysat uses the template string stored in
@@ -637,22 +733,36 @@ def update_data_directory_structure(new_template, test_run=True,
     return
 
 
-def check_and_make_path(path):
-    """Checks if path exists, creates it if it doesn't.
+def check_and_make_path(path, expand_path=False):
+    """Check if path exists and create it if needed.
 
     Parameters
     ----------
-    path : string
+    path : str
         Directory path without any file names. Creates all
         necessary directories to complete the path.
+    expand_path : bool
+        If True, input `path` will be processed through `os.path.expanduser`
+        and `os.path.expandvars`.
+
+    Returns
+    -------
+    made_dir : bool
+        True, if new directory made. False, if path already existed.
 
     Raises
     ------
     ValueError
-        If input path and internally constructed path are not equal.
+        If input path and internally constructed path are not equal, or
+        if an invalid path supplied.
 
     """
 
+    if expand_path:
+        # Account for home references, multi-platform
+        path = os.path.expanduser(path)
+        path = os.path.expandvars(path)
+
     if not os.path.exists(path):
         # Make path, checking to see that each level exists before attempting
         root_path, local_dir = os.path.split(path)
@@ -670,6 +780,10 @@ def check_and_make_path(path):
                 make_dir.append(local_dir)
             root_path, local_dir = os.path.split(root_path)
 
+            # Check that we continue to have a remotely valid path
+            if len(root_path) == 0:
+                raise ValueError('Invalid path specification.')
+
         if len(local_dir) > 0:
             # Avoid case where input is path='/stuff/level/'.
             # The trailing '/' leads to a local_dir=''
@@ -678,9 +792,72 @@ def check_and_make_path(path):
         while len(make_dir) > 0:
             local_dir = make_dir.pop()
             root_path = os.path.join(root_path, local_dir)
-            os.mkdir(root_path)
+            if (local_dir != '..') and (local_dir != '.'):
+                # Deal with case of path='... /path1/../final_path' or
+                # path='... /path1/./final_path'
+                os.mkdir(root_path)
 
         if os.path.normpath(root_path) != os.path.normpath(path):
-            raise ValueError('Desired and constructed paths differ')
+            estr = ''.join(['Desired and constructed paths unexpectedly differ',
+                            '. Please post an issue at https://github.com/pysa',
+                            't/pysat/issues'])
+            raise ValueError(estr)
 
-    return
+        return True
+    else:
+        return False
+
+
+def get_file_information(paths, root_dir=''):
+    """Create a dict with values from `os.stat` attributes for input path(s).
+
+    Parameters
+    ----------
+    paths : str or list
+        Full pathnames of files to get attribute information.
+    root_dir : str
+        Common root path shared by all paths, if any. (default='')
+
+    Returns
+    -------
+    file_info : dict
+        Keyed by file attribute. Each attribute maps to a list
+        of values for each file in `paths`.
+
+    See Also
+    --------
+    os.stat : Get variety of file attributes
+
+    """
+
+    paths = listify(paths)
+
+    # Mapping of output key to the attribute name returned by `os.stat`
+    attrs = {'content_modified_time': 'st_mtime', 'mode': 'st_mode',
+             'size': 'st_size', 'inode': 'st_ino', 'device': 'st_dev',
+             'nlink': 'st_nlink', 'uid': 'st_uid', 'gid': 'st_gid',
+             'last_access_time': 'st_atime',
+             'metadata_update_time': 'st_ctime'}
+
+    # Initiliaze output dictionary.
+    file_info = {}
+    for attr in attrs.keys():
+        file_info[attr] = []
+
+    # Add common root directory to paths, if supplied.
+    if root_dir != '':
+        for i, path in enumerate(paths):
+            paths[i] = os.path.join(root_dir, path)
+
+    # Collect file attributes and store.
+    for path in paths:
+        info = os.stat(path)
+        for attr in attrs.keys():
+            file_info[attr].append(getattr(info, attrs[attr]))
+
+    # Convert times to datetimes.
+    for attr in ['content_modified_time', 'last_access_time',
+                 'metadata_update_time']:
+        file_info[attr] = pds.to_datetime(file_info[attr], unit='s')
+
+    return file_info
